@@ -128,6 +128,7 @@ static gboolean avif_context_try_load(struct avif_context * context, GError ** e
     if (ret != AVIF_RESULT_OK) {
         g_set_error(error, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_FAILED,
                     "Failed to convert YUV to RGB: %s", avifResultToString(ret));
+        g_object_unref(output);
         return FALSE;
     }
 
@@ -208,7 +209,7 @@ static gboolean avif_context_try_load(struct avif_context * context, GError ** e
     if (image->transformFlags & AVIF_TRANSFORM_IMIR) {
         GdkPixbuf *output_mirrored = NULL;
 
-        switch (image->imir.mode) {
+        switch (image->imir.axis) {
         case 0:
             output_mirrored = gdk_pixbuf_flip(output, FALSE);
             break;
@@ -236,6 +237,7 @@ static gboolean avif_context_try_load(struct avif_context * context, GError ** e
                             GDK_PIXBUF_ERROR,
                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
                             "Transformed AVIF has zero width or height");
+        g_object_unref(output);
         return FALSE;
     }
 
@@ -250,6 +252,12 @@ static gboolean avif_context_try_load(struct avif_context * context, GError ** e
         }
     }
 
+    if (image->icc.size != 0) {
+        gchar *icc_base64 = g_base64_encode((const guchar *)image->icc.data, image->icc.size);
+        gdk_pixbuf_set_option(output, "icc-profile", icc_base64);
+        g_free(icc_base64);
+    }
+
     if (context->pixbuf) {
         g_object_unref(context->pixbuf);
         context->pixbuf = NULL;
@@ -466,7 +474,7 @@ static gboolean avif_image_saver(FILE          *f,
     encoder->maxQuantizer = max_quantizer;
     encoder->minQuantizerAlpha = 0;
     encoder->maxQuantizerAlpha = alpha_quantizer;
-    encoder->speed = 7;
+    encoder->speed = 6;
 
     res = avifEncoderWrite(encoder, avif, &raw);
     avifEncoderDestroy(encoder);
