@@ -9,6 +9,8 @@
 # The git tag below is known to work, and will occasionally be updated. Feel
 # free to use a more recent commit.
 
+set -e
+
 if [ $# -ne 1 ]; then
   echo "Usage: ${0} <path_to_android_ndk>"
   exit 1
@@ -32,6 +34,7 @@ for abi in ${ABI_LIST}; do
     -DLIBGAV1_THREADPOOL_USE_STD_MUTEX=1 \
     -DLIBGAV1_ENABLE_EXAMPLES=0 \
     -DLIBGAV1_ENABLE_TESTS=0 \
+    -DLIBGAV1_MAX_BITDEPTH=12 \
     -DANDROID_ABI=${abi}
   ninja
   cd ..
