@@ -4,7 +4,7 @@
 #ifndef AVIF_INTERNAL_H
 #define AVIF_INTERNAL_H
 
-#include "avif/avif.h"
+#include "avif/avif.h" // IWYU pragma: export
 
 #ifdef __cplusplus
 extern "C" {
@@ -29,6 +29,14 @@ extern "C" {
             return ERR;       \
     } while (0)
 
+// Forward any error to the caller now or continue execution.
+#define AVIF_CHECKRES(A)                 \
+    do {                                 \
+        const avifResult result__ = (A); \
+        if (result__ != AVIF_RESULT_OK)  \
+            return result__;             \
+    } while (0)
+
 // ---------------------------------------------------------------------------
 // URNs and Content-Types
 
@@ -72,13 +80,27 @@ void avifArrayPush(void * arrayStruct, void * element);
 void avifArrayPop(void * arrayStruct);
 void avifArrayDestroy(void * arrayStruct);
 
+void avifFractionSimplify(avifFraction * f);
+avifBool avifFractionCD(avifFraction * a, avifFraction * b);
+avifBool avifFractionAdd(avifFraction a, avifFraction b, avifFraction * result);
+avifBool avifFractionSub(avifFraction a, avifFraction b, avifFraction * result);
+
+void avifImageSetDefaults(avifImage * image);
+// Copies all fields that do not need to be freed/allocated from srcImage to dstImage.
+void avifImageCopyNoAlloc(avifImage * dstImage, const avifImage * srcImage);
+
+// Copies the samples from srcImage to dstImage. dstImage must be allocated.
+// srcImage and dstImage must have the same width, height, and depth.
+// If the AVIF_PLANES_YUV bit is set in planes, then srcImage and dstImage must have the same yuvFormat and yuvRange.
+void avifImageCopySamples(avifImage * dstImage, const avifImage * srcImage, avifPlanesFlags planes);
+
 typedef struct avifAlphaParams
 {
     uint32_t width;
     uint32_t height;
 
     uint32_t srcDepth;
-    uint8_t * srcPlane;
+    const uint8_t * srcPlane;
     uint32_t srcRowBytes;
     uint32_t srcOffsetBytes;
     uint32_t srcPixelBytes;
@@ -91,14 +113,18 @@ typedef struct avifAlphaParams
 
 } avifAlphaParams;
 
-avifBool avifFillAlpha(const avifAlphaParams * const params);
-avifBool avifReformatAlpha(const avifAlphaParams * const params);
+avifBool avifFillAlpha(const avifAlphaParams * params);
+avifBool avifReformatAlpha(const avifAlphaParams * params);
 
 typedef enum avifReformatMode
 {
     AVIF_REFORMAT_MODE_YUV_COEFFICIENTS = 0, // Normal YUV conversion using coefficients
     AVIF_REFORMAT_MODE_IDENTITY,             // Pack GBR directly into YUV planes (AVIF_MATRIX_COEFFICIENTS_IDENTITY)
-    AVIF_REFORMAT_MODE_YCGCO                 // YUV conversion using AVIF_MATRIX_COEFFICIENTS_YCGCO
+    AVIF_REFORMAT_MODE_YCGCO,                // YUV conversion using AVIF_MATRIX_COEFFICIENTS_YCGCO
+#if defined(AVIF_ENABLE_EXPERIMENTAL_YCGCO_R)
+    AVIF_REFORMAT_MODE_YCGCO_RE, // YUV conversion using AVIF_MATRIX_COEFFICIENTS_YCGCO_RE
+    AVIF_REFORMAT_MODE_YCGCO_RO, // YUV conversion using AVIF_MATRIX_COEFFICIENTS_YCGCO_RO
+#endif
 } avifReformatMode;
 
 typedef enum avifAlphaMultiplyMode
@@ -117,7 +143,6 @@ typedef struct avifReformatState
 
     uint32_t yuvChannelBytes;
     uint32_t rgbChannelBytes;
-    uint32_t rgbChannelCount;
     uint32_t rgbPixelBytes;
     uint32_t rgbOffsetBytesR;
     uint32_t rgbOffsetBytesG;
@@ -136,13 +161,7 @@ typedef struct avifReformatState
 
     avifPixelFormatInfo formatInfo;
 
-    // LUTs for going from YUV limited/full unorm -> full range RGB FP32
-    float unormFloatTableY[1 << 12];
-    float unormFloatTableUV[1 << 12];
-
     avifReformatMode mode;
-    // Used by avifImageYUVToRGB() only. avifImageRGBToYUV() uses a local variable (alphaMode) instead.
-    avifAlphaMultiplyMode toRGBAlphaMode;
 } avifReformatState;
 
 // Returns:
@@ -151,11 +170,19 @@ typedef struct avifReformatState
 // * [any other error]           - Return error to caller
 avifResult avifImageRGBToYUVLibYUV(avifImage * image, const avifRGBImage * rgb);
 
+// Parameters:
+// * image - input YUV image
+// * rgb - output RGB image
+// * reformatAlpha - if set to AVIF_TRUE, the function will attempt to copy the alpha channel to the output RGB image using
+// libyuv.
+// * alphaReformattedWithLibYUV - Output parameter. If reformatAlpha is set to true and libyuv was able to copy over the alpha
+// channel, then this will be set to AVIF_TRUE. Otherwise, this will be set to AVIF_FALSE. The value in this parameter is valid
+// only if the return value of the function is AVIF_RESULT_OK or AVIF_RESULT_NOT_IMPLEMENTED.
 // Returns:
 // * AVIF_RESULT_OK              - Converted successfully with libyuv
 // * AVIF_RESULT_NOT_IMPLEMENTED - The fast path for this combination is not implemented with libyuv, use built-in YUV conversion
 // * [any other error]           - Return error to caller
-avifResult avifImageYUVToRGBLibYUV(const avifImage * image, avifRGBImage * rgb);
+avifResult avifImageYUVToRGBLibYUV(const avifImage * image, avifRGBImage * rgb, avifBool reformatAlpha, avifBool * alphaReformattedWithLibYUV);
 
 // Returns:
 // * AVIF_RESULT_OK              - Converted successfully with libsharpyuv
@@ -197,6 +224,15 @@ avifBool avifImageScale(avifImage * image,
                         uint32_t imageDimensionLimit,
                         avifDiagnostics * diag);
 
+// ---------------------------------------------------------------------------
+// AVIF item type
+
+typedef enum avifItemCategory
+{
+    AVIF_ITEM_COLOR = 0,
+    AVIF_ITEM_ALPHA = 1
+} avifItemCategory;
+
 // ---------------------------------------------------------------------------
 // Grid AVIF images
 
@@ -207,8 +243,6 @@ avifBool avifAreGridDimensionsValid(avifPixelFormat yuvFormat, uint32_t imageW,
 // ---------------------------------------------------------------------------
 // Metadata
 
-// Validates the first bytes of the Exif payload and finds the TIFF header offset.
-avifResult avifGetExifTiffHeaderOffset(const avifRWData * exif, uint32_t * offset);
 // Attempts to parse the image->exif payload for Exif orientation and sets image->transformFlags, image->irot and
 // image->imir on success. Returns AVIF_RESULT_INVALID_EXIF_PAYLOAD on failure.
 avifResult avifImageExtractExifOrientationToIrotImir(avifImage * image);
@@ -237,8 +271,8 @@ AVIF_ARRAY_DECLARE(avifDecodeSampleArray, avifDecodeSample, sample);
 typedef struct avifCodecDecodeInput
 {
     avifDecodeSampleArray samples;
-    avifBool allLayers; // if true, the underlying codec must decode all layers, not just the best layer
-    avifBool alpha;     // if true, this is decoding an alpha plane
+    avifBool allLayers;            // if true, the underlying codec must decode all layers, not just the best layer
+    avifItemCategory itemCategory; // category of item being decoded
 } avifCodecDecodeInput;
 
 avifCodecDecodeInput * avifCodecDecodeInputCreate(void);
@@ -260,7 +294,7 @@ typedef struct avifCodecEncodeOutput
 } avifCodecEncodeOutput;
 
 avifCodecEncodeOutput * avifCodecEncodeOutputCreate(void);
-void avifCodecEncodeOutputAddSample(avifCodecEncodeOutput * encodeOutput, const uint8_t * data, size_t len, avifBool sync);
+avifResult avifCodecEncodeOutputAddSample(avifCodecEncodeOutput * encodeOutput, const uint8_t * data, size_t len, avifBool sync);
 void avifCodecEncodeOutputDestroy(avifCodecEncodeOutput * encodeOutput);
 
 // ---------------------------------------------------------------------------
@@ -272,13 +306,31 @@ typedef struct avifCodecSpecificOption
     char * value; // Free-form string to be interpreted by the codec
 } avifCodecSpecificOption;
 AVIF_ARRAY_DECLARE(avifCodecSpecificOptions, avifCodecSpecificOption, entries);
+
+// Returns NULL if a memory allocation failed.
 avifCodecSpecificOptions * avifCodecSpecificOptionsCreate(void);
 void avifCodecSpecificOptionsClear(avifCodecSpecificOptions * csOptions);
 void avifCodecSpecificOptionsDestroy(avifCodecSpecificOptions * csOptions);
-void avifCodecSpecificOptionsSet(avifCodecSpecificOptions * csOptions, const char * key, const char * value); // if(value==NULL), key is deleted
+avifResult avifCodecSpecificOptionsSet(avifCodecSpecificOptions * csOptions, const char * key, const char * value); // if(value==NULL), key is deleted
 
 // ---------------------------------------------------------------------------
-// avifCodec (abstraction layer to use different AV1 implementations)
+// avifCodecType (underlying video format)
+
+// Alliance for Open Media video formats that can be used in the AVIF image format.
+typedef enum avifCodecType
+{
+    AVIF_CODEC_TYPE_UNKNOWN,
+    AVIF_CODEC_TYPE_AV1,
+#if defined(AVIF_CODEC_AVM)
+    AVIF_CODEC_TYPE_AV2, // Experimental.
+#endif
+} avifCodecType;
+
+// Returns AVIF_CODEC_TYPE_UNKNOWN unless the chosen codec is available with the requiredFlags.
+avifCodecType avifCodecTypeFromChoice(avifCodecChoice choice, avifCodecFlags requiredFlags);
+
+// ---------------------------------------------------------------------------
+// avifCodec (abstraction layer to use different codec implementations)
 
 struct avifCodec;
 struct avifCodecInternal;
@@ -291,6 +343,9 @@ typedef enum avifEncoderChange
     AVIF_ENCODER_CHANGE_MAX_QUANTIZER_ALPHA = (1u << 3),
     AVIF_ENCODER_CHANGE_TILE_ROWS_LOG2 = (1u << 4),
     AVIF_ENCODER_CHANGE_TILE_COLS_LOG2 = (1u << 5),
+    AVIF_ENCODER_CHANGE_QUANTIZER = (1u << 6),
+    AVIF_ENCODER_CHANGE_QUANTIZER_ALPHA = (1u << 7),
+    AVIF_ENCODER_CHANGE_SCALING_MODE = (1u << 8),
 
     AVIF_ENCODER_CHANGE_CODEC_SPECIFIC = (1u << 31)
 } avifEncoderChange;
@@ -310,6 +365,10 @@ typedef avifBool (*avifCodecGetNextImageFunc)(struct avifCodec * codec,
 // encoder->tileRowsLog2, encoder->tileColsLog2, and encoder->autoTiling. The caller of
 // avifCodecEncodeImageFunc is responsible for automatic tiling if encoder->autoTiling is set to
 // AVIF_TRUE. The actual tiling values are passed to avifCodecEncodeImageFunc as parameters.
+// Similarly, avifCodecEncodeImageFunc should use the quantizer parameter instead of
+// encoder->quality and encoder->qualityAlpha. If disableLaggedOutput is AVIF_TRUE, then the encoder will emit the output frame
+// without any lag (if supported). Note that disableLaggedOutput is only used by the first call to this function (which
+// initializes the encoder) and is ignored by the subsequent calls.
 //
 // Note: The caller of avifCodecEncodeImageFunc always passes encoder->data->tileRowsLog2 and
 // encoder->data->tileColsLog2 as the tileRowsLog2 and tileColsLog2 arguments. Because
@@ -322,7 +381,9 @@ typedef avifResult (*avifCodecEncodeImageFunc)(struct avifCodec * codec,
                                                avifBool alpha,
                                                int tileRowsLog2,
                                                int tileColsLog2,
+                                               int quantizer,
                                                avifEncoderChanges encoderChanges,
+                                               avifBool disableLaggedOutput,
                                                avifAddImageFlags addImageFlags,
                                                avifCodecEncodeOutput * output);
 typedef avifBool (*avifCodecEncodeFinishFunc)(struct avifCodec * codec, avifCodecEncodeOutput * output);
@@ -359,6 +420,8 @@ avifCodec * avifCodecCreateRav1e(void);   // requires AVIF_CODEC_RAV1E (codec_ra
 const char * avifCodecVersionRav1e(void); // requires AVIF_CODEC_RAV1E (codec_rav1e.c)
 avifCodec * avifCodecCreateSvt(void);     // requires AVIF_CODEC_SVT (codec_svt.c)
 const char * avifCodecVersionSvt(void);   // requires AVIF_CODEC_SVT (codec_svt.c)
+avifCodec * avifCodecCreateAVM(void);     // requires AVIF_CODEC_AVM (codec_avm.c)
+const char * avifCodecVersionAVM(void);   // requires AVIF_CODEC_AVM (codec_avm.c)
 
 // ---------------------------------------------------------------------------
 // avifDiagnostics
@@ -377,14 +440,25 @@ typedef size_t avifBoxMarker;
 
 typedef struct avifBoxHeader
 {
+    // Size of the box in bytes, excluding the box header.
     size_t size;
+
     uint8_t type[4];
 } avifBoxHeader;
 
 typedef struct avifROStream
 {
     avifROData * raw;
+
+    // Index of the next byte in the raw stream.
     size_t offset;
+
+    // If 0, byte-aligned functions can be used (avifROStreamRead() etc.).
+    // Otherwise, it represents the number of bits already used in the last byte
+    // (located at offset-1).
+    size_t numUsedBitsInPartialByte;
+
+    // Error information, if any.
     avifDiagnostics * diag;
     const char * diagContext;
 } avifROStream;
@@ -396,6 +470,7 @@ void avifROStreamSetOffset(avifROStream * stream, size_t offset);
 
 avifBool avifROStreamHasBytesLeft(const avifROStream * stream, size_t byteCount);
 size_t avifROStreamRemainingBytes(const avifROStream * stream);
+// The following functions require byte alignment.
 avifBool avifROStreamSkip(avifROStream * stream, size_t byteCount);
 avifBool avifROStreamRead(avifROStream * stream, uint8_t * data, size_t size);
 avifBool avifROStreamReadU16(avifROStream * stream, uint16_t * v);
@@ -409,33 +484,48 @@ avifBool avifROStreamReadBoxHeader(avifROStream * stream, avifBoxHeader * header
 avifBool avifROStreamReadBoxHeaderPartial(avifROStream * stream, avifBoxHeader * header); // This doesn't require that the full box can fit in the stream
 avifBool avifROStreamReadVersionAndFlags(avifROStream * stream, uint8_t * version, uint32_t * flags); // version and flags ptrs are both optional
 avifBool avifROStreamReadAndEnforceVersion(avifROStream * stream, uint8_t enforcedVersion); // currently discards flags
+// The following functions can write non-aligned bits.
+avifBool avifROStreamReadBits8(avifROStream * stream, uint8_t * v, size_t bitCount);
+avifBool avifROStreamReadBits(avifROStream * stream, uint32_t * v, size_t bitCount);
+avifBool avifROStreamReadVarInt(avifROStream * stream, uint32_t * v);
 
 typedef struct avifRWStream
 {
     avifRWData * raw;
+
+    // Index of the next byte in the raw stream.
     size_t offset;
+
+    // If 0, byte-aligned functions can be used (avifRWStreamWrite() etc.).
+    // Otherwise, it represents the number of bits already used in the last byte
+    // (located at offset-1).
+    size_t numUsedBitsInPartialByte;
 } avifRWStream;
 
-uint8_t * avifRWStreamCurrent(avifRWStream * stream);
 void avifRWStreamStart(avifRWStream * stream, avifRWData * raw);
 size_t avifRWStreamOffset(const avifRWStream * stream);
 void avifRWStreamSetOffset(avifRWStream * stream, size_t offset);
 
 void avifRWStreamFinishWrite(avifRWStream * stream);
-void avifRWStreamWrite(avifRWStream * stream, const void * data, size_t size);
-void avifRWStreamWriteChars(avifRWStream * stream, const char * chars, size_t size);
-avifBoxMarker avifRWStreamWriteBox(avifRWStream * stream, const char * type, size_t contentSize);
-avifBoxMarker avifRWStreamWriteFullBox(avifRWStream * stream, const char * type, size_t contentSize, int version, uint32_t flags);
+// The following functions require byte alignment.
+avifResult avifRWStreamWrite(avifRWStream * stream, const void * data, size_t size);
+avifResult avifRWStreamWriteChars(avifRWStream * stream, const char * chars, size_t size);
+avifResult avifRWStreamWriteBox(avifRWStream * stream, const char * type, size_t contentSize, avifBoxMarker * marker);
+avifResult avifRWStreamWriteFullBox(avifRWStream * stream, const char * type, size_t contentSize, int version, uint32_t flags, avifBoxMarker * marker);
 void avifRWStreamFinishBox(avifRWStream * stream, avifBoxMarker marker);
-void avifRWStreamWriteU8(avifRWStream * stream, uint8_t v);
-void avifRWStreamWriteU16(avifRWStream * stream, uint16_t v);
-void avifRWStreamWriteU32(avifRWStream * stream, uint32_t v);
-void avifRWStreamWriteU64(avifRWStream * stream, uint64_t v);
-void avifRWStreamWriteZeros(avifRWStream * stream, size_t byteCount);
+avifResult avifRWStreamWriteU8(avifRWStream * stream, uint8_t v);
+avifResult avifRWStreamWriteU16(avifRWStream * stream, uint16_t v);
+avifResult avifRWStreamWriteU32(avifRWStream * stream, uint32_t v);
+avifResult avifRWStreamWriteU64(avifRWStream * stream, uint64_t v);
+avifResult avifRWStreamWriteZeros(avifRWStream * stream, size_t byteCount);
+// The following functions can write non-aligned bits.
+avifResult avifRWStreamWriteBits(avifRWStream * stream, uint32_t v, size_t bitCount);
+avifResult avifRWStreamWriteVarInt(avifRWStream * stream, uint32_t v);
 
 // This is to make it clear that the box size is currently unknown, and will be determined later (with a call to avifRWStreamFinishBox)
 #define AVIF_BOX_SIZE_TBD 0
 
+// Used for both av1C and av2C.
 typedef struct avifCodecConfigurationBox
 {
     // [skipped; is constant] unsigned int (1)marker = 1;
@@ -462,6 +552,7 @@ typedef struct avifCodecConfigurationBox
 
 typedef struct avifSequenceHeader
 {
+    uint8_t reduced_still_picture_header;
     uint32_t maxWidth;
     uint32_t maxHeight;
     uint32_t bitDepth;
@@ -471,9 +562,13 @@ typedef struct avifSequenceHeader
     avifTransferCharacteristics transferCharacteristics;
     avifMatrixCoefficients matrixCoefficients;
     avifRange range;
-    avifCodecConfigurationBox av1C;
+    avifCodecConfigurationBox av1C; // TODO(yguyon): Rename or add av2C
 } avifSequenceHeader;
-avifBool avifSequenceHeaderParse(avifSequenceHeader * header, const avifROData * sample);
+
+avifBool avifSequenceHeaderParse(avifSequenceHeader * header, const avifROData * sample, avifCodecType codecType);
+
+#define AVIF_INDEFINITE_DURATION64 UINT64_MAX
+#define AVIF_INDEFINITE_DURATION32 UINT32_MAX
 
 #ifdef __cplusplus
 } // extern "C"
